@@ -1,5 +1,6 @@
 #include "application.h"
 
+#include "appglobal.h"
 #include "actions/kcodecactionext.h"
 #include "actions/krecentfilesactionext.h"
 #include "actions/useraction.h"
@@ -8,13 +9,13 @@
 #include "gui/playerwidget.h"
 #include "gui/waveform/waveformwidget.h"
 #include "scripting/scriptsmanager.h"
+#include "translate/translatedialog.h"
 #include "videoplayer/videoplayer.h"
 
 #include <QAction>
 #include <QMenu>
 #include <QTextCodec>
 
-#include <KCharsets>
 #include <KLocalizedString>
 #include <kwidgetsaddons_version.h>
 
@@ -48,7 +49,7 @@ Application::setupActions()
 	openSubtitleAction->setText(i18nc("@action:inmenu Open subtitle file", "Open Subtitle..."));
 	openSubtitleAction->setStatusTip(i18n("Open subtitle file"));
 	actionCollection->setDefaultShortcuts(openSubtitleAction, KStandardShortcut::open());
-	connect(openSubtitleAction, &QAction::triggered, [this](){ openSubtitle(); });
+	connect(openSubtitleAction, &QAction::triggered, this, [this](){ openSubtitle(); });
 	actionCollection->addAction(ACT_OPEN_SUBTITLE, openSubtitleAction);
 	actionManager->addAction(openSubtitleAction, 0);
 
@@ -64,7 +65,7 @@ Application::setupActions()
 	m_recentSubtitlesAction->setIcon(QIcon::fromTheme("document-open"));
 	m_recentSubtitlesAction->setText(i18nc("@action:inmenu Open rencently used subtitle file", "Open &Recent Subtitle"));
 	m_recentSubtitlesAction->setStatusTip(i18n("Open subtitle file"));
-	connect(m_recentSubtitlesAction, &KRecentFilesActionExt::urlSelected, [this](const QUrl &url){ openSubtitle(url); });
+	connect(m_recentSubtitlesAction, &KRecentFilesActionExt::urlSelected, this, [this](const QUrl &url){ openSubtitle(url); });
 	actionCollection->addAction(ACT_RECENT_SUBTITLES, m_recentSubtitlesAction);
 
 	QAction *saveSubtitleAction = new QAction(actionCollection);
@@ -72,7 +73,7 @@ Application::setupActions()
 	saveSubtitleAction->setText(i18n("Save"));
 	saveSubtitleAction->setStatusTip(i18n("Save opened subtitle"));
 	actionCollection->setDefaultShortcuts(saveSubtitleAction, KStandardShortcut::save());
-	connect(saveSubtitleAction, &QAction::triggered, [this](){ saveSubtitle(KCharsets::charsets()->codecForName(m_subtitleEncoding)); });
+	connect(saveSubtitleAction, &QAction::triggered, this, [this](){ saveSubtitle(QTextCodec::codecForName(m_subtitleEncoding.toUtf8())); });
 	actionCollection->addAction(ACT_SAVE_SUBTITLE, saveSubtitleAction);
 	actionManager->addAction(saveSubtitleAction, UserAction::SubPDirty | UserAction::FullScreenOff);
 
@@ -108,7 +109,7 @@ Application::setupActions()
 	openSubtitleTrAction->setText(i18n("Open Translation..."));
 	openSubtitleTrAction->setStatusTip(i18n("Open translation subtitle file"));
 	actionCollection->setDefaultShortcut(openSubtitleTrAction, QKeySequence("Ctrl+Shift+O"));
-	connect(openSubtitleTrAction, &QAction::triggered, [this](){ openSubtitleTr(); });
+	connect(openSubtitleTrAction, &QAction::triggered, this, [this](){ openSubtitleTr(); });
 	actionCollection->addAction(ACT_OPEN_SUBTITLE_TR, openSubtitleTrAction);
 	actionManager->addAction(openSubtitleTrAction, UserAction::SubOpened);
 
@@ -124,7 +125,7 @@ Application::setupActions()
 	m_recentSubtitlesTrAction->setIcon(QIcon::fromTheme("document-open"));
 	m_recentSubtitlesTrAction->setText(i18n("Open &Recent Translation"));
 	m_recentSubtitlesTrAction->setStatusTip(i18n("Open translation subtitle file"));
-	connect(m_recentSubtitlesTrAction, &KRecentFilesActionExt::urlSelected, [this](const QUrl &url){ openSubtitleTr(url); });
+	connect(m_recentSubtitlesTrAction, &KRecentFilesActionExt::urlSelected, this, [this](const QUrl &url){ openSubtitleTr(url); });
 	actionCollection->addAction(ACT_RECENT_SUBTITLES_TR, m_recentSubtitlesTrAction);
 	actionManager->addAction(m_recentSubtitlesTrAction, UserAction::SubOpened | UserAction::FullScreenOff);
 
@@ -133,8 +134,8 @@ Application::setupActions()
 	saveSubtitleTrAction->setText(i18n("Save Translation"));
 	saveSubtitleTrAction->setStatusTip(i18n("Save opened translation subtitle"));
 	actionCollection->setDefaultShortcut(saveSubtitleTrAction, QKeySequence("Ctrl+Shift+S"));
-	connect(saveSubtitleTrAction, &QAction::triggered, [&](){
-		saveSubtitleTr(KCharsets::charsets()->codecForName(m_subtitleTrEncoding));
+	connect(saveSubtitleTrAction, &QAction::triggered, this, [&](){
+		saveSubtitleTr(QTextCodec::codecForName(m_subtitleTrEncoding.toUtf8()));
 	});
 	actionCollection->addAction(ACT_SAVE_SUBTITLE_TR, saveSubtitleTrAction);
 	actionManager->addAction(saveSubtitleTrAction, UserAction::SubSDirty | UserAction::FullScreenOff);
@@ -157,12 +158,12 @@ Application::setupActions()
 	actionCollection->addAction(ACT_CLOSE_SUBTITLE_TR, closeSubtitleTrAction);
 	actionManager->addAction(closeSubtitleTrAction, UserAction::SubTrOpened | UserAction::FullScreenOff);
 
-	QAction *undoAction = m_undoStack->undoAction();
+	QAction *undoAction = appUndoStack()->undoAction();
 	undoAction->setIcon(QIcon::fromTheme("edit-undo"));
 	actionCollection->setDefaultShortcuts(undoAction, KStandardShortcut::undo());
 	actionCollection->addAction(ACT_UNDO, undoAction);
 
-	QAction *redoAction = m_undoStack->redoAction();
+	QAction *redoAction = appUndoStack()->redoAction();
 	redoAction->setIcon(QIcon::fromTheme("edit-redo"));
 	actionCollection->setDefaultShortcuts(redoAction, KStandardShortcut::redo());
 	actionCollection->addAction(ACT_REDO, redoAction);
@@ -497,7 +498,7 @@ Application::setupActions()
 	QAction *translateAction = new QAction(actionCollection);
 	translateAction->setText(i18n("Translate..."));
 	translateAction->setStatusTip(i18n("Translate lines texts using Google services"));
-	connect(translateAction, &QAction::triggered, this, &Application::translate);
+	QObject::connect(translateAction, &QAction::triggered, this, &TranslateDialog::performTranslation);
 	actionCollection->addAction(ACT_TRANSLATE, translateAction);
 	actionManager->addAction(translateAction, UserAction::SubHasLine | UserAction::FullScreenOff);
 
@@ -505,7 +506,7 @@ Application::setupActions()
 	editCurrentLineInPlaceAction->setText(i18n("Edit Line in Place"));
 	editCurrentLineInPlaceAction->setStatusTip(i18n("Edit current line text in place"));
 	actionCollection->setDefaultShortcut(editCurrentLineInPlaceAction, QKeySequence("F2"));
-	connect(editCurrentLineInPlaceAction, &QAction::triggered, m_linesWidget, &LinesWidget::editCurrentLineInPlace);
+	connect(editCurrentLineInPlaceAction, &QAction::triggered, m_mainWindow->m_linesWidget, &LinesWidget::editCurrentLineInPlace);
 	actionCollection->addAction(ACT_EDIT_CURRENT_LINE_IN_PLACE, editCurrentLineInPlaceAction);
 	actionManager->addAction(editCurrentLineInPlaceAction, UserAction::HasSelection | UserAction::FullScreenOff);
 
@@ -513,32 +514,28 @@ Application::setupActions()
 	openVideoAction->setIcon(QIcon::fromTheme("document-open"));
 	openVideoAction->setText(i18n("Open Video..."));
 	openVideoAction->setStatusTip(i18n("Open video file"));
-	connect(openVideoAction, &QAction::triggered, [this](){ openVideo(); });
+	connect(openVideoAction, &QAction::triggered, this, [this](){ openVideo(); });
 	actionCollection->addAction(ACT_OPEN_VIDEO, openVideoAction);
 
 	m_recentVideosAction = new KRecentFilesActionExt(actionCollection);
 	m_recentVideosAction->setIcon(QIcon::fromTheme("document-open"));
 	m_recentVideosAction->setText(i18n("Open &Recent Video"));
 	m_recentVideosAction->setStatusTip(i18n("Open video file"));
-	connect(m_recentVideosAction, &KRecentFilesActionExt::urlSelected, [this](const QUrl &url){ openVideo(url); });
+	connect(m_recentVideosAction, &KRecentFilesActionExt::urlSelected, this, [this](const QUrl &url){ openVideo(url); });
 	actionCollection->addAction(ACT_RECENT_VIDEOS, m_recentVideosAction);
 
 	QAction *demuxTextStreamAction = new KSelectAction(actionCollection);
-	QMenu *demuxTextStreamActionMenu = new QMenu(m_mainWindow);
-	demuxTextStreamAction->setMenu(demuxTextStreamActionMenu);
 	demuxTextStreamAction->setIcon(QIcon::fromTheme("select_stream"));
 	demuxTextStreamAction->setText(i18n("Import Subtitle Stream"));
 	demuxTextStreamAction->setStatusTip(i18n("Import subtitle stream into subtitle editor"));
-	connect(demuxTextStreamActionMenu, &QMenu::triggered, [this](QAction *action){ demuxTextStream(action->data().value<int>()); });
+	connect(demuxTextStreamAction->menu(), &QMenu::triggered, this, [this](QAction *action){ demuxTextStream(action->data().value<int>()); });
 	actionCollection->addAction(ACT_DEMUX_TEXT_STREAM, demuxTextStreamAction);
 
 	QAction *speechImportStreamAction = new KSelectAction(actionCollection);
-	QMenu *speechImportStreamActionMenu = new QMenu(m_mainWindow);
-	speechImportStreamAction->setMenu(speechImportStreamActionMenu);
 	speechImportStreamAction->setIcon(QIcon::fromTheme("select_stream"));
 	speechImportStreamAction->setText(i18n("Recognize Speech"));
 	speechImportStreamAction->setStatusTip(i18n("Recognize speech in audio stream"));
-	connect(speechImportStreamActionMenu, &QMenu::triggered, [this](QAction *action){ speechImportAudioStream(action->data().value<int>()); });
+	connect(speechImportStreamAction->menu(), &QMenu::triggered, this, [this](QAction *action){ speechImportAudioStream(action->data().value<int>()); });
 	actionCollection->addAction(ACT_ASR_IMPORT_AUDIO_STREAM, speechImportStreamAction);
 
 	QAction *closeVideoAction = new QAction(actionCollection);
@@ -727,7 +724,7 @@ Application::setupActions()
 	increaseSubtitleFontAction->setText(i18n("Increase Font Size"));
 	increaseSubtitleFontAction->setStatusTip(i18n("Increase subtitles font size by 1 point"));
 	actionCollection->setDefaultShortcut(increaseSubtitleFontAction, QKeySequence("Alt++"));
-	connect(increaseSubtitleFontAction, &QAction::triggered, m_playerWidget, &PlayerWidget::increaseFontSize);
+	connect(increaseSubtitleFontAction, &QAction::triggered, m_mainWindow->m_playerWidget, &PlayerWidget::increaseFontSize);
 	actionCollection->addAction(ACT_INCREASE_SUBTITLE_FONT, increaseSubtitleFontAction);
 
 	QAction *decreaseSubtitleFontAction = new QAction(actionCollection);
@@ -735,7 +732,7 @@ Application::setupActions()
 	decreaseSubtitleFontAction->setText(i18n("Decrease Font Size"));
 	decreaseSubtitleFontAction->setStatusTip(i18n("Decrease subtitles font size by 1 point"));
 	actionCollection->setDefaultShortcut(decreaseSubtitleFontAction, QKeySequence("Alt+-"));
-	connect(decreaseSubtitleFontAction, &QAction::triggered, m_playerWidget, &PlayerWidget::decreaseFontSize);
+	connect(decreaseSubtitleFontAction, &QAction::triggered, m_mainWindow->m_playerWidget, &PlayerWidget::decreaseFontSize);
 	actionCollection->addAction(ACT_DECREASE_SUBTITLE_FONT, decreaseSubtitleFontAction);
 
 	KSelectAction *setActiveSubtitleStreamAction = new KSelectAction(actionCollection);
