@@ -1,23 +1,25 @@
 /*
     SPDX-FileCopyrightText: 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
-    SPDX-FileCopyrightText: 2010-2019 Mladen Milinkovic <max@smoothware.net>
+    SPDX-FileCopyrightText: 2010-2022 Mladen Milinkovic <max@smoothware.net>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#include "core/richdocument.h"
-#include "core/subtitle.h"
+#include "subtitle.h"
+
+#include "appglobal.h"
+#include "scconfig.h"
+#include "core/richtext/richdocument.h"
 #include "core/subtitleline.h"
 #include "core/subtitleiterator.h"
 #include "core/undo/subtitleactions.h"
 #include "core/undo/subtitlelineactions.h"
 #include "core/undo/undostack.h"
 #include "helpers/objectref.h"
-#include "scconfig.h"
-#include "application.h"
 #include "gui/treeview/lineswidget.h"
 
 #include <QTextDocumentFragment>
+#include <QTextEdit>
 
 #include <KLocalizedString>
 
@@ -43,6 +45,7 @@ Subtitle::Subtitle(double framesPerSecond)
 	  m_secondaryDirtyState(false),
 	  m_secondaryCleanIndex(0),
 	  m_framesPerSecond(framesPerSecond),
+	  m_stylesheet(new RichCSS()),
 	  m_formatData(nullptr)
 {}
 
@@ -50,6 +53,7 @@ Subtitle::~Subtitle()
 {
 	qDeleteAll(m_lines);
 
+	delete m_stylesheet;
 	delete m_formatData;
 }
 
@@ -58,6 +62,12 @@ Subtitle::setPrimaryData(const Subtitle &from, bool usePrimaryData)
 {
 	beginCompositeAction(i18n("Set Primary Data"));
 
+	m_metaData = from.m_metaData;
+
+	delete m_stylesheet;
+	m_stylesheet = new RichCSS(*from.m_stylesheet);
+	m_stylesheet->setParent(this);
+
 	setFormatData(from.m_formatData);
 
 	setFramesPerSecond(from.framesPerSecond());
@@ -75,6 +85,8 @@ Subtitle::setPrimaryData(const Subtitle &from, bool usePrimaryData)
 		thisLine->setTimes(fromLine->showTime(), fromLine->hideTime());
 		thisLine->setErrorFlags((fromLine->errorFlags() & fromErrors) | (thisLine->errorFlags() & thisErrors));
 		thisLine->setFormatData(fromLine->formatData());
+		thisLine->m_metaData = fromLine->m_metaData;
+		thisLine->m_position = fromLine->m_position;
 	}
 
 	if(fromIt.current()) { // from has more lines
@@ -85,6 +97,8 @@ Subtitle::setPrimaryData(const Subtitle &from, bool usePrimaryData)
 			thisLine->setPrimaryDoc(usePrimaryData ? cur->primaryDoc() : cur->secondaryDoc());
 			thisLine->setErrorFlags(SubtitleLine::SecondaryOnlyErrors, false);
 			thisLine->setFormatData(cur->formatData());
+			thisLine->m_metaData = cur->m_metaData;
+			thisLine->m_position = cur->m_position;
 			lines.append(thisLine);
 		}
 		processAction(new InsertLinesAction(this, lines));
@@ -92,7 +106,9 @@ Subtitle::setPrimaryData(const Subtitle &from, bool usePrimaryData)
 		for(SubtitleLine *thisLine = thisIt.current(); thisLine; ++thisIt, thisLine = thisIt.current()) {
 			thisLine->primaryDoc()->clear();
 			thisLine->setErrorFlags(SubtitleLine::PrimaryOnlyErrors, false);
-			thisLine->setFormatData(0);
+			thisLine->setFormatData(nullptr);
+			thisLine->m_metaData.clear();
+			thisLine->m_position = SubtitleRect();
 		}
 	}
 
@@ -169,7 +185,7 @@ Subtitle::clearPrimaryDirty()
 		return;
 
 	m_primaryDirtyState = false;
-	m_primaryCleanIndex = app()->undoStack()->index();
+	m_primaryCleanIndex = appUndoStack()->index();
 	emit primaryDirtyStateChanged(false);
 }
 
@@ -180,7 +196,7 @@ Subtitle::clearSecondaryDirty()
 		return;
 
 	m_secondaryDirtyState = false;
-	m_secondaryCleanIndex = app()->undoStack()->index();
+	m_secondaryCleanIndex = appUndoStack()->index();
 	emit secondaryDirtyStateChanged(false);
 }
 
@@ -195,7 +211,7 @@ Subtitle::setFormatData(const FormatData *formatData)
 {
 	delete m_formatData;
 
-	m_formatData = formatData ? new FormatData(*formatData) : NULL;
+	m_formatData = formatData ? new FormatData(*formatData) : nullptr;
 }
 
 double
@@ -389,7 +405,7 @@ Subtitle::insertNewLine(int index, bool insertAfter, SubtitleTarget target)
 		SubtitleLine *line = newLine;
 		SubtitleIterator it(*this, Range::full(), false);
 		for(it.toIndex(newLineIndex + 1); it.current(); ++it) {
-			line->setSecondaryDoc(it.current()->secondaryDoc());
+			line->secondaryDoc()->setRichText(it.current()->secondaryDoc()->toRichText());
 			line = it.current();
 		}
 		line->secondaryDoc()->clear();
@@ -403,7 +419,7 @@ Subtitle::insertNewLine(int index, bool insertAfter, SubtitleTarget target)
 		SubtitleIterator it(*this, Range::full(), true);
 		SubtitleLine *line = it.current();
 		for(--it; it.index() >= index; --it) {
-			line->setSecondaryDoc(it.current()->secondaryDoc());
+			line->secondaryDoc()->setRichText(it.current()->secondaryDoc()->toRichText());
 			line = it.current();
 		}
 		line->secondaryDoc()->clear();
@@ -448,7 +464,7 @@ Subtitle::removeLines(const RangeList &r, SubtitleTarget target)
 		SubtitleIterator srcIt(*this, rangesComplement);
 		SubtitleIterator dstIt(*this, Range::upper(ranges.firstIndex()));
 		for(; srcIt.current() && dstIt.current(); ++srcIt, ++dstIt)
-			dstIt.current()->setSecondaryDoc(srcIt.current()->secondaryDoc());
+			dstIt.current()->secondaryDoc()->setRichText(srcIt.current()->secondaryDoc()->toRichText());
 
 		// the remaining lines secondary text must be cleared
 		for(; dstIt.current(); ++dstIt)
@@ -487,7 +503,7 @@ Subtitle::removeLines(const RangeList &r, SubtitleTarget target)
 		SubtitleIterator srcIt(*this, Range(ranges.firstIndex(), m_lines.count() - lines.count() - 1), true);
 		SubtitleIterator dstIt(*this, rangesComplement, true);
 		for(; srcIt.current() && dstIt.current(); --srcIt, --dstIt)
-			dstIt.current()->setSecondaryDoc(srcIt.current()->secondaryDoc());
+			dstIt.current()->secondaryDoc()->setRichText(srcIt.current()->secondaryDoc()->toRichText());
 
 		// finally, we can remove the specified lines
 		RangeList::ConstIterator rangesIt = ranges.end(), begin = ranges.begin();
@@ -525,7 +541,7 @@ Subtitle::splitLines(const RangeList &ranges)
 				c->insertText(QString(QChar::LineFeed));
 				return true;
 			}
-			if(text.at(j) == QChar::Space) {
+			if(j < len && text.at(j) == QChar::Space) {
 				c->movePosition(QTextCursor::Start);
 				c->movePosition(QTextCursor::NextCharacter, QTextCursor::MoveAnchor, j);
 				c->movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
@@ -1219,7 +1235,8 @@ Subtitle::splitSubtitle(Subtitle &dstSubtitle, const Time &splitTime, bool shift
 	}
 
 	if(splitIndex > 0 || (splitIndex == 0 && splitsLine)) {
-		dstSubtitle.m_formatData = m_formatData ? new FormatData(*m_formatData) : 0;
+		dstSubtitle.m_stylesheet = m_stylesheet ? new RichCSS(*m_stylesheet) : new RichCSS();
+		dstSubtitle.m_formatData = m_formatData ? new FormatData(*m_formatData) : nullptr;
 
 		dstSubtitle.beginCompositeAction(i18n("Split Subtitles"));
 		if(dstSubtitle.count())
@@ -1240,7 +1257,7 @@ Subtitle::splitSubtitle(Subtitle &dstSubtitle, const Time &splitTime, bool shift
 }
 
 void
-Subtitle::toggleStyleFlag(const RangeList &ranges, SString::StyleFlag styleFlag)
+Subtitle::toggleStyleFlag(const RangeList &ranges, RichString::StyleFlag styleFlag)
 {
 	SubtitleIterator it(*this, ranges);
 	if(!it.current())
@@ -1250,19 +1267,19 @@ Subtitle::toggleStyleFlag(const RangeList &ranges, SString::StyleFlag styleFlag)
 
 	QTextCharFormat fmtPri, fmtSec;
 	switch(styleFlag) {
-	case SString::Bold:
+	case RichString::Bold:
 		fmtPri.setFontWeight(QTextCursor(it.current()->primaryDoc()).charFormat().fontWeight() == QFont::Bold ? QFont::Normal : QFont::Bold);
 		fmtSec.setFontWeight(QTextCursor(it.current()->secondaryDoc()).charFormat().fontWeight() == QFont::Bold ? QFont::Normal : QFont::Bold);
 		break;
-	case SString::Italic:
+	case RichString::Italic:
 		fmtPri.setFontItalic(!QTextCursor(it.current()->primaryDoc()).charFormat().fontItalic());
 		fmtSec.setFontItalic(!QTextCursor(it.current()->secondaryDoc()).charFormat().fontItalic());
 		break;
-	case SString::Underline:
+	case RichString::Underline:
 		fmtPri.setFontUnderline(!QTextCursor(it.current()->primaryDoc()).charFormat().fontUnderline());
 		fmtSec.setFontUnderline(!QTextCursor(it.current()->secondaryDoc()).charFormat().fontUnderline());
 		break;
-	case SString::StrikeThrough:
+	case RichString::StrikeThrough:
 		fmtPri.setFontStrikeOut(!QTextCursor(it.current()->primaryDoc()).charFormat().fontStrikeOut());
 		fmtSec.setFontStrikeOut(!QTextCursor(it.current()->secondaryDoc()).charFormat().fontStrikeOut());
 		break;
@@ -1368,37 +1385,37 @@ Subtitle::recheckErrors(const RangeList &ranges)
 void
 Subtitle::processAction(UndoAction *action) const
 {
-	if(app()->subtitle() == this)
-		app()->undoStack()->push(action);
-	else
+	if(appSubtitle() == this) {
+		appUndoStack()->push(action);
+	} else {
 		action->redo();
+		delete action;
+	}
 }
 
 void
 Subtitle::beginCompositeAction(const QString &title) const
 {
-	if(app()->subtitle() == this)
-		app()->undoStack()->beginMacro(title);
+	if(appSubtitle() == this)
+		appUndoStack()->beginMacro(title);
 }
 
 void
 Subtitle::endCompositeAction(UndoStack::DirtyMode dirtyOverride) const
 {
-	if(app()->subtitle() == this)
-		app()->undoStack()->endMacro(dirtyOverride);
+	if(appSubtitle() == this)
+		appUndoStack()->endMacro(dirtyOverride);
 }
 
 bool
 Subtitle::isPrimaryDirty(int index) const
 {
-	const UndoStack *undoStack = app()->undoStack();
-
 	int i = m_primaryCleanIndex;
 	const int d = i > index ? -1 : 1;
 	for(;;) {
 		if(i < 0)
 			return m_primaryCleanIndex >= 0;
-		const UndoStack::DirtyMode dirtyMode = i > 0 ? undoStack->dirtyMode(i - 1) : UndoStack::None;
+		const UndoStack::DirtyMode dirtyMode = i > 0 ? appUndoStack()->dirtyMode(i - 1) : UndoStack::None;
 		if(i != m_primaryCleanIndex && (dirtyMode & UndoStack::Primary))
 			return true;
 		if(i == index)
@@ -1410,14 +1427,12 @@ Subtitle::isPrimaryDirty(int index) const
 bool
 Subtitle::isSecondaryDirty(int index) const
 {
-	const UndoStack *undoStack = app()->undoStack();
-
 	int i = m_secondaryCleanIndex;
 	const int d = i > index ? -1 : 1;
 	for(;;) {
 		if(i < 0)
 			return m_secondaryCleanIndex >= 0;
-		const UndoStack::DirtyMode dirtyMode = i > 0 ? undoStack->dirtyMode(i - 1) : UndoStack::None;
+		const UndoStack::DirtyMode dirtyMode = i > 0 ? appUndoStack()->dirtyMode(i - 1) : UndoStack::None;
 		if(i != m_secondaryCleanIndex && (dirtyMode & UndoStack::Secondary))
 			return true;
 		if(i == index)
@@ -1429,9 +1444,8 @@ Subtitle::isSecondaryDirty(int index) const
 void
 Subtitle::updateState()
 {
-	const UndoStack *undoStack = app()->undoStack();
-	const int index = undoStack->index();
-	const UndoStack::DirtyMode dirtyMode = index > 0 ? undoStack->dirtyMode(index - 1) : UndoStack::Both;
+	const int index = appUndoStack()->index();
+	const UndoStack::DirtyMode dirtyMode = index > 0 ? appUndoStack()->dirtyMode(index - 1) : UndoStack::Both;
 
 	if(m_primaryDirtyState != isPrimaryDirty(index)) {
 		m_primaryDirtyState = !m_primaryDirtyState;
@@ -1448,6 +1462,26 @@ Subtitle::updateState()
 		emit secondaryChanged();
 }
 
+void
+Subtitle::stylesheetEdit(QTextEdit *textEdit)
+{
+	if(m_ignoreDocChanges)
+		return;
+	processAction(new EditStylesheetAction(this, textEdit));
+}
+
+void
+Subtitle::stylesheetAppend(const QString &css)
+{
+	m_stylesheet->parse(css);
+}
+
+void
+Subtitle::stylesheetClear()
+{
+	m_stylesheet->clear();
+}
+
 /// SUBTITLECOMPOSITEACTIONEXECUTOR
 
 SubtitleCompositeActionExecutor::SubtitleCompositeActionExecutor(const Subtitle *subtitle, const QString &title)
