@@ -1,11 +1,11 @@
 /*
     SPDX-FileCopyrightText: 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
-    SPDX-FileCopyrightText: 2010-2019 Mladen Milinkovic <max@smoothware.net>
+    SPDX-FileCopyrightText: 2010-2022 Mladen Milinkovic <max@smoothware.net>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#include "core/richdocument.h"
+#include "core/richtext/richdocument.h"
 #include "core/subtitleline.h"
 #include "core/undo/subtitlelineactions.h"
 #include "core/undo/subtitleactions.h"
@@ -21,15 +21,6 @@
 using namespace SubtitleComposer;
 
 
-/// "magic" code taken from https://tekpool.wordpress.com/category/bit-count/
-int
-SubtitleLine::bitsCount(unsigned int u)
-{
-	unsigned int uCount;
-	uCount = u - ((u >> 1) & 033333333333) - ((u >> 2) & 011111111111);
-	return ((uCount + (uCount >> 3)) & 030707070707) % 63;
-}
-
 SubtitleLine::ErrorFlag
 SubtitleLine::errorFlag(SubtitleLine::ErrorID id)
 {
@@ -257,7 +248,7 @@ SubtitleLine::index() const
 void
 SubtitleLine::primaryDocumentChanged()
 {
-	if(m_ignoreDocChanges)
+	if(m_ignoreDocChanges || (m_subtitle && m_subtitle->m_ignoreDocChanges))
 		return;
 	processAction(new SetLinePrimaryTextAction(this, m_primaryDoc));
 }
@@ -265,7 +256,7 @@ SubtitleLine::primaryDocumentChanged()
 void
 SubtitleLine::secondaryDocumentChanged()
 {
-	if(m_ignoreDocChanges)
+	if(m_ignoreDocChanges || (m_subtitle && m_subtitle->m_ignoreDocChanges))
 		return;
 	processAction(new SetLineSecondaryTextAction(this, m_secondaryDoc));
 }
@@ -273,20 +264,30 @@ SubtitleLine::secondaryDocumentChanged()
 void
 SubtitleLine::setPrimaryDoc(RichDocument *doc)
 {
-	if(m_primaryDoc)
+	if(m_primaryDoc == doc)
+		return;
+	if(m_primaryDoc) {
 		disconnect(m_primaryDoc, &RichDocument::contentsChanged, this, &SubtitleLine::primaryDocumentChanged);
+		m_primaryDoc->setStylesheet(nullptr);
+	}
 	m_primaryDoc = doc;
 	m_primaryDoc->setParent(this);
+	m_primaryDoc->setStylesheet(m_subtitle ? m_subtitle->m_stylesheet : nullptr);
 	connect(m_primaryDoc, &RichDocument::contentsChanged, this, &SubtitleLine::primaryDocumentChanged);
 }
 
 void
 SubtitleLine::setSecondaryDoc(RichDocument *doc)
 {
-	if(m_secondaryDoc)
+	if(m_secondaryDoc == doc)
+		return;
+	if(m_secondaryDoc) {
 		disconnect(m_secondaryDoc, &RichDocument::contentsChanged, this, &SubtitleLine::secondaryDocumentChanged);
+		m_secondaryDoc->setStylesheet(nullptr);
+	}
 	m_secondaryDoc = doc;
 	m_secondaryDoc->setParent(this);
+	m_secondaryDoc->setStylesheet(m_subtitle ? m_subtitle->m_stylesheet : nullptr);
 	connect(m_secondaryDoc, &RichDocument::contentsChanged, this, &SubtitleLine::secondaryDocumentChanged);
 }
 
@@ -321,14 +322,14 @@ SubtitleLine::unbreakText(SubtitleTarget target)
 {
 	switch(target) {
 	case Primary:
-		m_primaryDoc->replace('\n', ' ');
+		m_primaryDoc->joinLines();
 		break;
 	case Secondary:
-		m_secondaryDoc->replace('\n', ' ');
+		m_secondaryDoc->joinLines();
 		break;
 	case Both:
-		m_primaryDoc->replace('\n', ' ');
-		m_secondaryDoc->replace('\n', ' ');
+		m_primaryDoc->joinLines();
+		m_secondaryDoc->joinLines();
 		break;
 	default:
 		break;
@@ -475,7 +476,7 @@ SubtitleLine::primaryWords() const
 int
 SubtitleLine::primaryLines() const
 {
-	SString text(m_primaryDoc->toPlainText());
+	RichString text(m_primaryDoc->toPlainText());
 	text.simplifyWhiteSpace();
 	return text.isEmpty() ? 0 : text.count('\n') + 1;
 }
@@ -496,7 +497,7 @@ SubtitleLine::secondaryWords() const
 int
 SubtitleLine::secondaryLines() const
 {
-	SString text(m_secondaryDoc->toPlainText());
+	RichString text(m_secondaryDoc->toPlainText());
 	text.simplifyWhiteSpace();
 	return text.isEmpty() ? 0 : text.count('\n') + 1;
 }
@@ -508,7 +509,7 @@ SubtitleLine::autoDuration(const QString &t, int msecsPerChar, int msecsPerWord,
 	Q_ASSERT(msecsPerWord >= 0);
 	Q_ASSERT(msecsPerLine >= 0);
 
-	SString text(t);
+	RichString text(t);
 	text.simplifyWhiteSpace();
 	if(text.isEmpty())
 		return 0;
@@ -556,6 +557,13 @@ SubtitleLine::adjustTimes(double shiftMseconds, double scaleFactor)
 			i18n("Adjust Line Times")));
 }
 
+void
+SubtitleLine::setPosition(const SubtitleRect &pos)
+{
+	m_position = pos;
+	emit positionChanged();
+}
+
 /// ERRORS
 
 int
@@ -567,7 +575,7 @@ SubtitleLine::errorFlags() const
 int
 SubtitleLine::errorCount() const
 {
-	return bitsCount(m_errorFlags);
+	return bitCount(quint32(m_errorFlags));
 }
 
 void
@@ -586,9 +594,7 @@ SubtitleLine::setErrorFlags(int errorFlags, bool value)
 bool
 SubtitleLine::checkEmptyPrimaryText(bool update)
 {
-	static const QRegExp emptyTextRegExp("^\\s*$");
-
-	bool error = m_primaryDoc->isEmpty() || m_primaryDoc->toPlainText().indexOf(emptyTextRegExp) != -1;
+	bool error = m_primaryDoc->isEmpty() || m_primaryDoc->toPlainText().trimmed().isEmpty();
 
 	if(update)
 		setErrorFlags(EmptyPrimaryText, error);
@@ -599,9 +605,7 @@ SubtitleLine::checkEmptyPrimaryText(bool update)
 bool
 SubtitleLine::checkEmptySecondaryText(bool update)
 {
-	static const QRegExp emptyTextRegExp("^\\s*$");
-
-	bool error = m_secondaryDoc->isEmpty() || m_secondaryDoc->toPlainText().indexOf(emptyTextRegExp) != -1;
+	bool error = m_secondaryDoc->isEmpty() || m_secondaryDoc->toPlainText().trimmed().isEmpty();
 
 	if(update)
 		setErrorFlags(EmptySecondaryText, error);
@@ -795,63 +799,65 @@ SubtitleLine::checkSecondaryUnneededSpaces(bool update)
 bool
 SubtitleLine::checkPrimaryCapitalAfterEllipsis(bool update)
 {
-	static const QRegExp capitalAfterEllipsisRegExp("^\\s*\\.\\.\\.[¡¿\\.,;\\(\\[\\{\"'\\s]*");
+	staticRE$(capitalAfterEllipsisRegExp, "^\\s*\\.\\.\\.[¡¿\\.,;\\(\\[\\{\"'\\s]*", REu);
 
 	const QString text = m_primaryDoc->toPlainText();
-	bool error = capitalAfterEllipsisRegExp.indexIn(text, QRegExp::CaretAtZero) != -1;
-	if(error) {
-		QChar chr = text.at(capitalAfterEllipsisRegExp.matchedLength());
-		error = chr.isLetter() && chr == chr.toUpper();
+	QRegularExpressionMatchIterator it = capitalAfterEllipsisRegExp.globalMatch(text);
+	bool success = it.hasNext();
+	if(success) {
+		const QChar chr = text.at(it.next().capturedEnd());
+		success = chr.isLetter() && chr == chr.toUpper();
 	}
 
 	if(update)
-		setErrorFlags(PrimaryCapitalAfterEllipsis, error);
+		setErrorFlags(PrimaryCapitalAfterEllipsis, success);
 
-	return error;
+	return success;
 }
 
 bool
 SubtitleLine::checkSecondaryCapitalAfterEllipsis(bool update)
 {
-	static const QRegExp capitalAfterEllipsisRegExp("^\\s*\\.\\.\\.[¡¿\\.,;\\(\\[\\{\"'\\s]*");
+	staticRE$(capitalAfterEllipsisRegExp, "^\\s*\\.\\.\\.[¡¿\\.,;\\(\\[\\{\"'\\s]*", REu);
 
 	const QString text = m_secondaryDoc->toPlainText();
-	bool error = capitalAfterEllipsisRegExp.indexIn(text, QRegExp::CaretAtZero) != -1;
-	if(error) {
-		QChar chr = text.at(capitalAfterEllipsisRegExp.matchedLength());
-		error = chr.isLetter() && chr == chr.toUpper();
+	QRegularExpressionMatchIterator it = capitalAfterEllipsisRegExp.globalMatch(text);
+	bool success = it.hasNext();
+	if(success) {
+		const QChar chr = text.at(it.next().capturedEnd());
+		success = chr.isLetter() && chr == chr.toUpper();
 	}
 
 	if(update)
-		setErrorFlags(SecondaryCapitalAfterEllipsis, error);
+		setErrorFlags(SecondaryCapitalAfterEllipsis, success);
 
-	return error;
+	return success;
 }
 
 bool
 SubtitleLine::checkPrimaryUnneededDash(bool update)
 {
-	static const QRegExp unneededDashRegExp("(^|\n)\\s*-[^-]");
+	staticRE$(unneededDashRegExp, "(^|\n)\\s*-[^-]", REu);
 
-	bool error = m_primaryDoc->toPlainText().count(unneededDashRegExp) == 1;
+	bool success = m_primaryDoc->toPlainText().count(unneededDashRegExp) == 1;
 
 	if(update)
-		setErrorFlags(PrimaryUnneededDash, error);
+		setErrorFlags(PrimaryUnneededDash, success);
 
-	return error;
+	return success;
 }
 
 bool
 SubtitleLine::checkSecondaryUnneededDash(bool update)
 {
-	static const QRegExp unneededDashRegExp("(^|\n)\\s*-[^-]");
+	staticRE$(unneededDashRegExp, "(^|\n)\\s*-[^-]", REu);
 
-	bool error = m_secondaryDoc->toPlainText().count(unneededDashRegExp) == 1;
+	bool success = m_secondaryDoc->toPlainText().count(unneededDashRegExp) == 1;
 
 	if(update)
-		setErrorFlags(SecondaryUnneededDash, error);
+		setErrorFlags(SecondaryUnneededDash, success);
 
-	return error;
+	return success;
 }
 
 int
