@@ -1,6 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
-    SPDX-FileCopyrightText: 2010-2019 Mladen Milinkovic <max@smoothware.net>
+    SPDX-FileCopyrightText: 2010-2022 Mladen Milinkovic <max@smoothware.net>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -17,15 +17,15 @@ class InputFormat : public Format
 public:
 	bool readSubtitle(Subtitle &subtitle, bool primary, const QString &data) const
 	{
-		Subtitle newSubtitle;
+		QExplicitlySharedDataPointer<Subtitle> newSubtitle(new Subtitle());
 
-		if(!parseSubtitles(newSubtitle, data))
+		if(!parseSubtitles(*newSubtitle, data))
 			return false;
 
 		if(primary)
-			subtitle.setPrimaryData(newSubtitle, true);
+			subtitle.setPrimaryData(*newSubtitle, true);
 		else
-			subtitle.setSecondaryData(newSubtitle, true);
+			subtitle.setSecondaryData(*newSubtitle, true);
 
 		return true;
 	}
