@@ -1,11 +1,12 @@
 /*
-    SPDX-FileCopyrightText: 2010-2021 Mladen Milinkovic <max@smoothware.net>
+    SPDX-FileCopyrightText: 2010-2022 Mladen Milinkovic <max@smoothware.net>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "waveformwidget.h"
 
+#include "appglobal.h"
 #include "application.h"
 #include "scconfig.h"
 #include "core/subtitleline.h"
@@ -67,7 +68,7 @@ WaveformWidget::WaveformWidget(QWidget *parent)
 	  m_zoomDataLen(0)
 {
 	m_widgetLayout = new QBoxLayout(QBoxLayout::LeftToRight);
-	m_widgetLayout->setMargin(0);
+	m_widgetLayout->setContentsMargins(0, 0, 0, 0);
 	m_widgetLayout->setSpacing(0);
 
 	m_waveformGraphics->installEventFilter(this);
@@ -89,7 +90,7 @@ WaveformWidget::WaveformWidget(QWidget *parent)
 	m_btnAutoScroll = createToolButton(QStringLiteral(ACT_WAVEFORM_AUTOSCROLL));
 
 	QHBoxLayout *toolbarLayout = new QHBoxLayout();
-	toolbarLayout->setMargin(0);
+	toolbarLayout->setContentsMargins(0, 0, 0, 0);
 	toolbarLayout->setSpacing(2);
 	toolbarLayout->addWidget(m_btnZoomOut);
 	toolbarLayout->addWidget(m_btnZoomIn);
@@ -101,7 +102,7 @@ WaveformWidget::WaveformWidget(QWidget *parent)
 	m_toolbar->setLayout(toolbarLayout);
 
 	QBoxLayout *mainLayout = new QVBoxLayout(this);
-	mainLayout->setMargin(0);
+	mainLayout->setContentsMargins(0, 0, 0, 0);
 	mainLayout->setSpacing(5);
 	mainLayout->addWidget(m_toolbar);
 	mainLayout->addLayout(m_widgetLayout);
@@ -191,12 +192,12 @@ WaveformWidget::setZoom(quint32 val)
 
 	m_wfBuffer->zoomBuffer()->setZoomScale(val);
 
-	const quint32 samMS = m_wfBuffer->sampleRateMillis();
-	if(samMS) {
-		const qint32 msSpanOld = m_zoom * span / samMS;
+	const quint32 samRt = m_wfBuffer->sampleRate();
+	if(samRt) {
+		const qint32 msSpanOld = m_zoom * span * 1000 / samRt;
 		m_zoom = val;
 
-		const quint32 msSpanNew = val * span / samMS;
+		const quint32 msSpanNew = val * span * 1000 / samRt;
 		m_timeStart.shift((msSpanOld - qint32(msSpanNew)) / 2);
 		m_timeEnd = m_timeStart.shifted(msSpanNew);
 		handleTimeUpdate(msSpanNew);
@@ -238,9 +239,9 @@ WaveformWidget::handleTimeUpdate(quint32 msSpan)
 void
 WaveformWidget::onWaveformResize(quint32 span)
 {
-	const quint32 samMS = m_wfBuffer->sampleRateMillis();
-	if(samMS) {
-		const double windowSize = m_zoom * span / samMS;
+	const quint32 samRt = m_wfBuffer->sampleRate();
+	if(samRt) {
+		const double windowSize = m_zoom * span * 1000 / samRt;
 		m_timeEnd = m_timeStart.shifted(windowSize);
 		handleTimeUpdate(windowSize);
 		updateActions();
@@ -370,19 +371,38 @@ WaveformWidget::updateVisibleLines()
 
 	auto it = m_visibleLines.begin();
 	while(it != m_visibleLines.end()) {
-		if(*it != m_draggedLine)
+		if(*it == m_draggedLine) {
+			it = m_visibleLines.erase(it);
+		} else if((*it)->line()->subtitle() != m_subtitle.data()
+			|| !(*it)->line()->intersectsTimespan(m_timeStart, m_timeEnd)) {
 			delete *it;
-		++it;
+			it = m_visibleLines.erase(it);
+		} else {
+			++it;
+		}
 	}
-	m_visibleLines.clear();
 
-	const SubtitleLine *dragged = m_draggedLine ? m_draggedLine->line() : nullptr;
+	it = m_visibleLines.begin();
 	for(int i = 0, n = m_subtitle->count(); i < n; i++) {
 		SubtitleLine *sub = m_subtitle->at(i);
-		if(sub == dragged)
-			m_visibleLines.push_back(m_draggedLine);
-		else if(sub->intersectsTimespan(m_timeStart, m_timeEnd))
-			m_visibleLines.push_back(new WaveSubtitle(sub, m_waveformGraphics));
+		const bool isDragged = m_draggedLine != nullptr && sub == m_draggedLine->line();
+		if(!sub->intersectsTimespan(m_timeStart, m_timeEnd) && !isDragged)
+			continue;
+		const Time showTime = isDragged ? m_draggedLine->showTime() : sub->showTime();
+		while(it != m_visibleLines.end() && (*it)->showTime() < showTime) {
+			if((*it)->line() == sub)
+				break;
+			++it;
+		}
+		if(it == m_visibleLines.end() || (*it)->line() != sub) {
+			if(isDragged) {
+				m_visibleLines.emplace(it, m_draggedLine);
+				it = m_visibleLines.begin();
+			} else {
+				it = m_visibleLines.emplace(it, new WaveSubtitle(sub, m_waveformGraphics));
+				++it;
+			}
+		}
 	}
 }
 
@@ -438,14 +458,14 @@ WaveformWidget::draggableAt(double posTime, WaveSubtitle **result)
 	if(result)
 		*result = nullptr;
 
-	if(!m_wfBuffer->sampleRateMillis())
+	if(!m_wfBuffer->sampleRate())
 		return DRAG_NONE;
 
-	double msTolerance = double(10 * m_wfBuffer->millisPerPixel());
+	const double msTolerance = 10. * m_wfBuffer->zoomBuffer()->samplesPerPixel() * 1000. / m_wfBuffer->sampleRate();
 
 	DragPosition dragMode = DRAG_NONE;
-	foreach(WaveSubtitle *sub, m_visibleLines) {
-		DragPosition dm = sub->draggableAt(posTime, &msTolerance);
+	for(WaveSubtitle *sub: m_visibleLines) {
+		DragPosition dm = sub->draggableAt(posTime, msTolerance);
 		if(dm > DRAG_FORBIDDEN || dragMode == DRAG_NONE) {
 			dragMode = dm;
 			if(result && dm > DRAG_FORBIDDEN)
@@ -460,7 +480,7 @@ SubtitleLine *
 WaveformWidget::subtitleLineAtMousePosition() const
 {
 	const Time mouseTime = m_RMBDown ? m_timeRMBRelease : m_pointerTime;
-	foreach(WaveSubtitle *sub, m_visibleLines) {
+	for(const WaveSubtitle *sub: qAsConst(m_visibleLines)) {
 		if(sub->line()->containsTime(mouseTime))
 			return sub->line();
 	}
@@ -559,12 +579,12 @@ WaveformWidget::mousePress(int pos, Qt::MouseButton button)
 }
 
 bool
-WaveformWidget::mouseRelease(int pos, Qt::MouseButton button, QPoint globalPos)
+WaveformWidget::mouseRelease(int pos, Qt::MouseButton button, const QPointF &globalPos)
 {
 	if(button == Qt::RightButton) {
 		m_timeRMBRelease = timeAt(pos);
 		m_hoverScrollTimer.stop();
-		showContextMenu(globalPos);
+		showContextMenu(globalPos.toPoint());
 		m_RMBDown = false;
 		return false;
 	}
@@ -635,7 +655,6 @@ WaveformWidget::onPlayerPositionChanged(double seconds)
 		if(m_autoScroll && !m_draggedLine && !m_autoScrollPause)
 			scrollToTime(m_timeCurrent, true);
 
-		m_visibleLinesDirty = true;
 		if(m_waveformGraphics)
 			m_waveformGraphics->update();
 	}
@@ -691,7 +710,7 @@ WaveformWidget::showContextMenu(QPoint pos)
 			}));
 		menu->addSeparator();
 		needSubtitle.append(
-			menu->addAction(i18n("Join Lines"), [&](){
+			menu->addAction(i18n("Join Lines"), this, [&](){
 				int startIndex = -1, endIndex = -1;
 				const Time startTime = rightMouseSoonerTime();
 				const Time endTime = rightMouseLaterTime();
@@ -709,13 +728,13 @@ WaveformWidget::showContextMenu(QPoint pos)
 			})
 		);
 		needCurrentLine.append(
-			menu->addAction(i18n("Split Line"), [&](){
+			menu->addAction(i18n("Split Line"), this, [&](){
 				// TODO: split the line at exact waveform mouse position
 				m_subtitle->splitLines(RangeList(Range(currentLine->index())));
 			}));
 		menu->addSeparator();
 		needCurrentLine.append(
-			menu->addAction(i18n("Toggle Anchor"), [&](){ m_subtitle->toggleLineAnchor(currentLine); }));
+			menu->addAction(i18n("Toggle Anchor"), this, [&](){ m_subtitle->toggleLineAnchor(currentLine); }));
 		menu->addAction(app->action(ACT_ANCHOR_REMOVE_ALL));
 		menu->addSeparator();
 		actionManager->addAction(
