@@ -1,6 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2007-2009 Sergio Pistone <sergio_pistone@yahoo.com.ar>
-    SPDX-FileCopyrightText: 2010-2019 Mladen Milinkovic <max@smoothware.net>
+    SPDX-FileCopyrightText: 2010-2022 Mladen Milinkovic <max@smoothware.net>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -14,6 +14,7 @@
 
 namespace SubtitleComposer {
 class PlayerWidget;
+class SubtitleMetaWidget;
 class LinesWidget;
 class CurrentLineWidget;
 class WaveformWidget;
@@ -33,14 +34,12 @@ public:
 
 	QMenu *createPopupMenu() override;
 
-public slots:
-	void setSubtitle(Subtitle *subtitle = 0);
-
 protected:
 	bool queryClose() override;
 
 protected:
 	PlayerWidget *m_playerWidget;
+	SubtitleMetaWidget *m_metaWidget;
 	LinesWidget *m_linesWidget;
 	CurrentLineWidget *m_curLineWidget;
 	WaveformWidget *m_waveformWidget;
