@@ -1,6 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2003 Fabrice Bellard
-    SPDX-FileCopyrightText: 2020 Mladen Milinkovic <max@smoothware.net>
+    SPDX-FileCopyrightText: 2020-2022 Mladen Milinkovic <max@smoothware.net>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -45,7 +45,7 @@ isRealTime(AVFormatContext *s)
 #else
 	const char *url = s->url;
 #endif
-	if(s->pb && (!strncmp(url, "rtp:", 4) || !strncmp(url, "udp:", 4)))
+	if(s->pb && url && (!strncmp(url, "rtp:", 4) || !strncmp(url, "udp:", 4)))
 		return 1;
 	return 0;
 }
@@ -230,7 +230,7 @@ StreamDemuxer::componentOpen(int streamIndex)
 {
 	AVFormatContext *ic = m_vs->fmtContext;
 	AVCodecContext *avCtx;
-	AVCodec *codec;
+	const AVCodec *codec;
 	AVDictionary *opts = nullptr;
 	AVDictionaryEntry *t = nullptr;
 	int sampleRate, nbChannels;
@@ -286,15 +286,13 @@ StreamDemuxer::componentOpen(int streamIndex)
 		av_dict_set(&opts, "threads", "auto", 0);
 	if(stream_lowres)
 		av_dict_set_int(&opts, "lowres", stream_lowres, 0);
-	if(avCtx->codec_type == AVMEDIA_TYPE_VIDEO || avCtx->codec_type == AVMEDIA_TYPE_AUDIO)
-		av_dict_set(&opts, "refcounted_frames", "1", 0);
 	if((ret = avcodec_open2(avCtx, codec, &opts)) < 0) {
+		char e[AV_ERROR_MAX_STRING_SIZE];
+		av_log(nullptr, AV_LOG_ERROR, "Failed opening codec err:%d - %s.\n", ret, av_make_error_string(e, sizeof(e), ret));
 		goto fail;
 	}
 	if((t = av_dict_get(opts, "", nullptr, AV_DICT_IGNORE_SUFFIX))) {
-		av_log(nullptr, AV_LOG_ERROR, "Option %s not found.\n", t->key);
-		ret = AVERROR_OPTION_NOT_FOUND;
-		goto fail;
+		av_log(nullptr, AV_LOG_WARNING, "AVCodec option %s not found.\n", t->key);
 	}
 
 	m_vs->eof = false;
@@ -506,7 +504,7 @@ StreamDemuxer::run()
 
 	{ // find_stream_info
 		const int origNbStreams = ic->nb_streams;
-		AVDictionary **opts = (AVDictionary **)av_mallocz_array(origNbStreams, sizeof(*opts));
+		AVDictionary **opts = (AVDictionary **)av_calloc(origNbStreams, sizeof(*opts));
 		if(!opts) {
 			av_log(nullptr, AV_LOG_ERROR, "Could not alloc memory for stream options.\n");
 			goto cleanup;
